/************************************************************************/
/* mc.h                                                                 */
/*                                                                      */
/* A MonteCarlo model for radiative transfer.                           */
/*                                                                      */
/* Author: Bernhard Mayer,                                              */
/*         NCAR, bmayer@ucar.edu                                        */
/*                                                                      */
/*----------------------------------------------------------------------*/
/* Copyright (C) 1999 Bernhard Mayer                                    */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*----------------------------------------------------------------------*/
/************************************************************************/

#ifndef __mc_h
#define __mc_h

#if defined (__cplusplus)
extern "C" {
#endif



typedef enum {NONE, UPPER, LOWER} BOUNDARY;

typedef struct {
  double a;
  double b;
  double c;
  double d;
  double upper;
} surface;


typedef struct {
  int n;
  double *air;
  double *aer;
  double *ozo;
  double *cld;
  double *ice;
  double *tot;
} profile;


/* profile3D is required to store the profiles of  */
/* absorption and scattering coefficients; float   */
/* should be accurate enough                       */

typedef struct {
  int Nz;
  int Nx;
  int Ny;
  int *threed;
  float ***cld;
  float ***tot;
} profile3D;



/* ddprofile3D is required to store the profile of */
/* the absorbed energy; as all output quantities,  */
/* this parameter is also stored in double rather  */
/* than float                                      */

typedef struct {
  int Nz;
  int Nx;
  int Ny;
  int *threed;
  /* double ***aer; */
  /* double ***ozo; */
  /* double ***ice; */
  double ***cld;
  double ***tot;
} ddprofile3D;

typedef struct {
  double air;
  double aer;
  double ozo;
  double cld;
  double ice;
  double tot;
} optical_depth;


typedef struct {
  double **edir;
  double **edn; 
  double **eup;  
  double **fdir;
  double **fdn; 
  double **fup;  
  double ***raddir;
  double ***raddif;
} result;


typedef struct {
  double dx[3];
  int hit[3];
  double cotheta; 
} direction;




/* prototypes */
int mc (int *nlyr,
		  float *dtair, float *dto3, 
		  float *dtaer, float *omaer, float *fgaer,
		  float *dtcld, float *omcld, float *fgcld,
		  float *dtice, float *omice, float *fgice,
		  float *zprof,
		  float *alb, float *sza, float *phi0, int *mcphotons,
		  float *rfldir, float *rfldn,  float *flup,
		  float *uavgso, float *uavgdn, float *uavgup);

static double sc_Isotropic_phi();

static double sc_Rayleigh_theta();

static double sc_Isotropic_theta();

static double sc_Lambertian_theta();

static double sc_HG_theta(double g);

static int calc_Legendre_table (char *filename, 
				double **F1, double **D1, 
				double **mu1, int *nF);

static double sc_Legendre_theta (double *F, double *D, double *mu, int nF);

static double random_tau();



#if defined (__cplusplus)
}
#endif

#endif






