/************************************************************************/
/* wcloud3d.h                                                           */
/*                                                                      */
/* Read 3D cloud descritpions.                                          */
/*                                                                      */
/* Author: Bernhard Mayer,                                              */
/*         NCAR, bmayer@ucar.edu                                        */
/*                                                                      */
/*----------------------------------------------------------------------*/
/* Copyright (C) 1999 Bernhard Mayer                                    */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*----------------------------------------------------------------------*/
/************************************************************************/

#ifndef __wcloud3d_h
#define __wcloud3_h

#if defined (__cplusplus)
extern "C" {
#endif
  
  
/* Structure to store a direction and opening angle */
/* for radiance calculations; convention:           */ 
/*    phi=0,   South                                */
/*    phi=90,  West                                 */
/*    phi=180, North                                */
/*    phi=270, East                                 */
/*                                                  */
/*    theta > 0 upward                              */
/*    theta < 0 downward                            */

typedef struct {
  double theta;
  double phi;
  double alpha; 
  double cosalpha; 
  double omega; 
  double dx[3];
} radang;


/* prototypes */
int read_3D_cloud (char *filename, 
		   int *Nx, int *Ny, int *Nz,
		   double *delX, double *delY,
		   double **zcld3D, float ****kcld3d, int **threed,
		   float ****gcld3d, int *gspec,
		   float ****wcld3d, int *wspec);

int read_2D_albedo (char *filename, 
		    int *Nx, int *Ny,
		    double *delX, double *delY,
		    double ***albedo);

int read_2D_elevation (char *filename, 
		       int *Nx, int *Ny,
		       double *delX, double *delY,
		       double ***elevation);

int read_2D_sample (char *filename, 
		    int *Nx, int *Ny, int *Nz,
		    double *delX, double *delY, double **z,
		    radang **rad, int *nrad);

#if defined (__cplusplus)
}
#endif

#endif






