/* mie_lex.l                                                -*- C -*-

Copyright (C) 1995 Arve Kylling

This file is part of libRadtran.

Radtran is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Radtran is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to the Free
Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/

%{

#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include "mie.h"

float  yytext2float(char* yytext, int which_token, int max_token);
float* yytext2floats(char* yytext, int nary);
int    yytext2nfloats(char* yytext);
int    yytext2int(char* yytext, int which_token, int max_token);
int*   yytext2integers(char* yytext, int nary);
int    yytext2nintegers(char* yytext);
char*  yytext2string(char* yytext, int which_token, int max_token);

char* s;
float* tmp;
int ierror=0;
int start_condition = INITIAL;
int line_number;
input_struct Input;
output_struct Output;

%}

%option noyywrap

DIGIT    [0-9]
SPACE    [ \t]
CCHAR    [#]
COMMENT  ({CCHAR}.*)
INTEGER  {DIGIT}+
SPACE_INTEGER {SPACE}+{INTEGER}
INTEGERS {SPACE_INTEGER}+ 
EXPON   ([DdEe][+-]?{DIGIT}+)
FLOAT    (({DIGIT}+"."{DIGIT}*)|({DIGIT}+"."{DIGIT}*{EXPON}))
SPACE_FLOAT {SPACE}+{FLOAT}
FLOATS   {SPACE_FLOAT}+ 
SIGN     [-+]
SIGNED_FLOAT {SIGN}?{FLOAT}
SPACE_SIGNED_FLOAT {SPACE}+{SIGNED_FLOAT}
SIGNED_FLOATS {SPACE_SIGNED_FLOAT}+
SIGNED_INTEGER {SIGN}?{INTEGER}
WORD     [_\.\/A-Za-z][_\.\/A-Za-z0-9\-]*

%%

  /* These are the allowed input lines in the uvspec input file 
     The input line specifications must come before the eating of
     blanks, otherwise things get messed up 
     Obviously this whole lot could have been handled more properly
     using bison etc., but than rather few would know what was going 
     on??? 
   */

refrac{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncmp("ice",s,3)==0) {Input.medium = ICE;}
				     free(s);}
mie_program{SPACE}+{WORD}           {s =  yytext2string(yytext,2,2);
				     if (strncmp("MIEV0",s,5)==0) {Input.program = MIEV0;}
				     else if (strncmp(s,"BH",2)==0) {Input.program = BHMIE;}}
mimcut{SPACE}+{SIGNED_FLOAT}        {Input.mie.mimcut   = yytext2float(yytext,2,2); }
r_mean{SPACE}+{SIGNED_FLOAT}        {Input.r_mean   = yytext2float(yytext,2,2); }
wvn{FLOATS}                         {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.wl.wl_start  = yytext2float(s,2,3); 
                                     Input.wl.wl_end    = yytext2float(yytext,3,3);
				     free(s);}
wvn_step{SPACE}+{SIGNED_FLOAT}      {Input.wl.wl_step   = yytext2float(yytext,2,2); }


{SPACE}+       ;         /* eat up blanks */ 

{COMMENT}*     ;         /* eat up comments */

\n                        { ++line_number;};

{WORD}                    {fprintf(stderr,"Unknown command on line %d\n", line_number+1);
                           fprintf(stderr,"%s\n", yytext);
                           ierror++;}

<<EOF>> { 
  if ( ierror > 0 ) { exit(1); }
	    yyterminate();
        }      

%%

/*************************************************************/
/* print usage information                                   */
/*************************************************************/

static void usage()
{
  fprintf (stderr, "\nCalculates the 1D radiation field of the Earth\'s atmosphere.\n");
  fprintf (stderr, "See libRadtran documentation for more information.\n\n");
  fprintf (stderr, "No error checking yet, so be careful.\n\n");

  fprintf (stderr, "Usage: uvspec [-h] < input_file > Output_file\n");
  fprintf (stderr, " -h    Print this message.\n");  
  fprintf (stderr, "\n");  
}

int main(int argc, char **argv) 
{
  
  /*  @40c@                

@code{mie} performes Mie scattering calculations for a specified
wavelength interval. It reads input from standard input, 
and outputs to standard output. It is normally invoked in the following
way: 

@example
mie < input_file > output_file
@end example

@noindent The format of the input and output files are described
below. Several realistic examples of input files are subsequently 
given.

@strong{Warning:} Please note the error checking on input variables 
is very scarce at the moment. Hence, if you provide erroneous input, 
the outcome is unpredictable.
  @c40@ */

  /*  @40_01c@                
@cindex input file, mie
@cindex mie, input file

The mie input file consists of single line entries, each making
up a complete input to the uvspec program. First on the line comes
the parameter name, followed by one or more parameter values. The 
parameter name and the parameter values are seperated by white space.

Filenames are entered without any surrounding single or double quotes.

Comments are introduced by a @code{#}. Blank lines are ignored.

The various input parameters are described in detail below.
@vtable @code
@item mie_program
Specify which Mie program to use:
@vtable @code
@item BH
The Mie scattering program from Bohren and Hoffmann.
@item MIEV0
The Mie scattering program due to W. Wiscombe @@
(ftp://climate.gsfc.nasa.gov/pub/wiscombe).
@end vtable
@item mimcut
(positive) value below which imaginary refractive
index is regarded as zero (computation proceeds
faster for zero imaginary index). Only valid for mie_program @code{MIEV0}.
@item r_mean
The mean radius of the particle to calculate single scattering properties of.
In micrometer. Used together with the wavelength information to calculate the 
Mie size parameter.
@item refrac
Specify which refractive index to use. The following options are implemented: 
@vtable @code
\item ice
The complex refractive index is taken from the REFICE function of W. Wiscombe. 
@end vtable
@item wvn
Sets the wavelength range, in nm. Specify first wavelength and last 
wavelength. The wavelength step is specified by @code{wvn_step}.
@item wvn_step
The wavelength step, in nm.

@end vtable
  @c40_01@ */

  /*  @40_02c@                
@cindex output, mie
@cindex uvspec, mie
The mie output is currently fixed. We hope to make it user controllable
in the future.

mie outputs one line to standard output (stdout) for each wavelength.
The format of the output line is

@example
lambda refrac_real refrac_img qext omega gg spike
@end example

Here

@vtable @code
@item lambda
Wavelength (nm)
@item refrac_real
@itemx refrac_img
The real and imaginary part of the refractive index respectively.
@item qext
The extinction efficiency factor.
@item omega
The single scattering albedo.
@item gg
The asymmetry factor.
@item spike
To quote from Wiscombe's @file{MIEV0.doc}:
     @quotation
(REAL) magnitude of the smallest denominator of
            either Mie coefficient (a-sub-n or b-sub-n),
            taken over all terms in the Mie series past
            N = size parameter XX.  Values of SPIKE below
            about 0.3 signify a ripple spike, since these
            spikes are produced by abnormally small denominators
            in the Mie coefficients (normal denominators are of
            order unity or higher).  Defaults to 1.0 when not
            on a spike.  Does not identify all resonances
            (we are still working on that).
     @end quotation
@end vtable

  @c40_02@ */

  /*  @40_03c@                

An example of a complete input file is

@example
@include ../examples/MIE_1.INP
@end example


      @c40_03@   */

  /***************** End of documentation *********************/

  int c;
  /* int iu, iv, j, lev; */
  int status=0;
  /*  Input_struct Input;
  Output_struct Output;*/

  /* Initialization */

  Output.print.phi    = 1; /* Default print radiances */
  Input.wl.wl_start     = 0.0;
  Input.wl.wl_end       = 0.0;
  Input.mie.anyang      = 0;
  Input.mie.ipolzn      = 0;
  Input.mie.momdim      = 1;
  Input.mie.nmom        = 0;
  Input.mie.numang      = 0;
  Input.mie.mimcut      = 1.0e-08;
  Input.mie.perfct      = 0;
  Input.mie.prnt[0]     = 0;
  Input.mie.prnt[1]     = 0;
  Input.mie.xmu         = (float *) calloc (Input.mie.numang+1, sizeof(float));
  Input.mie.s1          = (complex *) calloc (Input.mie.numang+1, sizeof(complex));
  Input.mie.s2          = (complex *) calloc (Input.mie.numang+1, sizeof(complex));

  while ((c=getopt (argc, argv, "h")) != EOF)  {
    switch(c)  {
    case 'h': 
      usage();
      return (-1);
      break;
    default:
      usage();
      return (-1);
    }
  }

  /* Read the input file */

  yylex();

  status = mie_check(Input);
  if (status!=0) {
    fprintf (stderr, "%d error(s) in uvspec input-file, aborting\n", status);
    return status;
  }

  /* Do the radiation calculation */

  status = mie(Input,&Output);
  if ( status != 0.0 ) {
    fprintf (stderr, "Error during execution of uvspec\n");
    return status;
  }
 
  /* Print output ***FIXME*** THIS SHOULD BE USER DEFINED SOMEHOW */

  return status;

}

float yytext2float(char* yytext, int which_token, int max_token) {
  double dnumber;
  char token[] = " ";
  char *p, *s=NULL;
  int count;

  p = strtok(yytext,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if ( count == which_token ) {
      s = p;
    }
  }
  if ( (count != which_token) && (count>max_token) ) {
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  }
  dnumber = atof(s);
  /*   printf("dnumner %f\n",dnumber);*/
  return (float)dnumber;
}

float* yytext2floats(char* yytext, int nary) {
  int i;
  char token[] = " ";
  char *p;

  /* Fill up ary */
  float *ary    = (float *) calloc (nary, sizeof(float));

  p = strtok(yytext,token);
  i = 0;
  while ( (p = strtok(NULL,token)) != NULL) {
    ary[i++] = atof(p);
  }
  return ary;
}

int yytext2nfloats(char* yytext) {
  int nary;
  char token[] = " ";
  char *p;

  /* Get number of ary elements */
  nary = 0;
  p = strtok(yytext,token);
  while ( (p = strtok(NULL,token)) != NULL) {
    nary++;
  }
  return nary;
}

int* yytext2integers(char* yytext, int nary) {
  int i;
  char token[] = " ";
  char *p;

  /* Fill up ary */
  int *ary    = (int *) calloc (nary, sizeof(int));

  p = strtok(yytext,token);
  i = 0;
  while ( (p = strtok(NULL,token)) != NULL) {
    ary[i++] = atoi(p);
  }
  return ary;
}

int yytext2nintegers(char* yytext) {
  int nary;
  char token[] = " ";
  char *p;

  /* Get number of ary elements */
  nary = 0;
  p = strtok(yytext,token);
  while ( (p = strtok(NULL,token)) != NULL) {
    nary++;
  }
  return nary;
}

int yytext2int(char* yytext, int which_token, int max_token) {
  int inumber;
  char token[] = " ";
  char *p, *s=NULL;
  int count;

  p = strtok(yytext,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if ( count == which_token ) {
      s = p;
    }
  }
  if ( (count != which_token) && (count>max_token) ) {
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  }
  inumber = atoi(s);
  /*  printf("inumner %d\n",inumber);*/
  return inumber;
}

char* yytext2string(char* yytext, int which_token, int max_token) {
  char token[] = " ";
  char *p, *s=NULL, *string=NULL;
  int count;

  p = strtok(yytext,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if ( count == which_token ) {
      s = p;
    }
  }
  if ( (count != which_token) && (count>max_token) ) {
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  }
  string = (char *) calloc (strlen(s)+1, sizeof (char));
  strcpy (string, s);
  return string;
}

