/************************************************************************/
/*                                                                      */
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*                                                                      */
/************************************************************************/

#include <stdio.h>							    
#include <stdlib.h>
#include "radiative.h"
#include "ascii.h"
#include "uvspec.h"

/************************************/
/* prototypes of internal functions */
/************************************/

int setup_crs (input_struct input, output_struct *output);

int crs_o3 (crs_out_struct *crs_out, int first, float *lambda, 
	   int n_lambda, int ntemp, char *path, float *temper,
	   int type);



/**************************************************************/
/* Setup absorption by molecules                              */
/**************************************************************/

int setup_crs (input_struct input, output_struct *output)
{
  int status = 0, iv;
  static int first = 1;
  
  /* Allocate memory for crs, the number 2 makes room for Rayleigh and ozone */
  /* cross sections, more may follow. */
  if ( first ) {
    status  = ASCII_calloc_float_3D(&output->crs.crs, output->atm.nlev, 
				    2, output->wl.nlambda_t );
    if (status!=0) { 
      fprintf (stderr, " ... error %d allocating memory\n", status); 
      return status; 
    } 
  }
  /* Get temperature dependent o3 cross section */
  status = crs_o3 (&output->crs,
		   first, output->wl.lambda_t, 
		   output->wl.nlambda_t, output->atm.nlev, input.filepath, 
		   output->atm.temper, input.crs.o3_type);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up o3 cross section\n", status);
    return status;
  }

  status = crs_rayleigh(output->wl.lambda_t, output->wl.nlambda_t, output->crs.crs);
  /*  for (iv=0;iv<output->wl.nlambda_t;iv++) {
    printf("crs %d %f %e %e\n", iv, output->wl.lambda_t[iv], output->crs.crs[0][AIR][iv], output->crs.crs[0][O3][iv]);
    }*/

  /* Get additional requested cross sections */
  /**** FIXME: YET TO BE IMPLEMENTED   ****/
  if ( strlen(input.crs.filename) > 0 ) {
    printf("crs: %d %s\n",input.crs.column,input.crs.filename);
    return -1;
  }

  first=0;
  return 0;
}






